@@ -1,11 +1,10 @@
 #
 # -*- Perl -*-
-# $Id: mhonarc.pl,v 1.23.8.5 2004/03/20 15:48:31 usu Exp $
+# $Id: mhonarc.pl,v 1.23.8.7 2005/06/06 05:41:42 opengl2772 Exp $
 # Copyright (C) 1997-2000 Satoru Takabayashi ,
-#               1999 NOKUBI Takatsugu,
-#               2002 Earl Hood,
-#               2000-2003 Namazu Project
-#               All rights reserved.
+#               1999 NOKUBI Takatsugu ,
+#               2002 Earl Hood ,
+#               2000-2005 Namazu Project All rights reserved.
 #     This is free software with ABSOLUTELY NO WARRANTY.
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -45,6 +44,10 @@ sub mediatype() {
 }
 
 sub status() {
+    # The check of a dependence filter.
+    return 'no' if (html::status() ne 'yes');
+    return 'no' if (mailnews::status() ne 'yes');
+
     return 'yes';
 }
 
@@ -157,18 +160,21 @@ sub load_mhonarc_fields {
 
     if ($mha_head =~ /<!--X-Subject: ([^-]+) -->/) {
 	my $subject = uncommentize($1);
-        codeconv::toeuc(\$subject);
+        # codeconv::toeuc(\$subject);
+        codeconv::codeconv_document(\$subject);
 	1  while ($subject =~ s/\A\s*(re|sv|fwd|fw)[\[\]\d]*[:>-]+\s*//i);
 	$subject =~ s/\A\s*\[[^\]]+\]\s*//;
 	$fields->{'subject'} = $subject;
     }
     if ($mha_head =~ /<!--X-From-R13: ([^-]+) -->/) {
         my $from = mrot13(uncommentize($1));
-        codeconv::toeuc(\$from);
+        # codeconv::toeuc(\$from);
+        codeconv::codeconv_document(\$from);
         $fields->{'from'} = $from;
     } elsif ($mha_head =~ /<!--X-From: ([^-]+) -->/) {
         my $from = uncommentize($1);
-        codeconv::toeuc(\$from);
+        # codeconv::toeuc(\$from);
+        codeconv::codeconv_document(\$from);
         $fields->{'from'} = $from;
     }
     if ($mha_head =~ /<!--X-Message-Id: ([^-]+) -->/) {
