@@ -1,8 +1,8 @@
 #
 # -*- Perl -*-
-# $Id: ooo.pl,v 1.4.2.7 2004/03/09 12:56:34 usu Exp $
+# $Id: ooo.pl,v 1.4.2.13 2007/01/28 20:24:04 opengl2772 Exp $
 # Copyright (C) 2003 Yukio USUDA 
-#               2003,2004 Namazu Project All rights reserved ,
+#               2003-2007 Namazu Project All rights reserved ,
 #     This is free software with ABSOLUTELY NO WARRANTY.
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -24,32 +24,36 @@
 
 package ooo;
 use strict;
+use English;
 require 'util.pl';
 require 'gfilter.pl';
 
 
-my $perlver =$];
 my $utfconvpath = undef;
 my $unzippath = undef;
-my $unzipopts;
+my @unzipopts;
 
 sub mediatype() {
     # http://framework.openoffice.org/documentation/mimetypes/mimetypes.html
     return ('application/vnd.sun.xml.writer',
             'application/vnd.sun.xml.calc',
             'application/vnd.sun.xml.impress',
-            'application/vnd.sun.xml.draw');
+            'application/vnd.sun.xml.draw',
+            'application/vnd.oasis.opendocument.text',
+            'application/vnd.oasis.opendocument.spreadsheet',
+            'application/vnd.oasis.opendocument.presentation',
+            'application/vnd.oasis.opendocument.graphics');
 }
 
 sub status() {
     $unzippath = util::checkcmd('unzip');
     if (defined $unzippath){
-	$unzipopts = ('-p');
-       if (util::islang("ja")) {
-           if ($perlver >= 5.008) {
-		$utfconvpath = "";
-		return 'yes';
-	   }
+        @unzipopts = ("-p");
+        if (util::islang("ja")) {
+           if ($English::PERL_VERSION >= 5.008) {
+               $utfconvpath = "none";
+               return 'yes';
+           }
            $utfconvpath = util::checkcmd('lv');
            if ($utfconvpath){ 
                return 'yes';
@@ -63,8 +67,8 @@ sub status() {
         } else {
            return 'yes'; 
         }
-    return 'no';
     }
+    return 'no';
 }
 
 sub recursive() {
@@ -81,10 +85,17 @@ sub post_codeconv () {
 
 sub add_magic ($) {
     my ($magic) = @_;
-    $magic->addFileExts('\\.sxw', 'application/vnd.sun.xml.writer');
-    $magic->addFileExts('\\.sxc', 'application/vnd.sun.xml.calc');
-    $magic->addFileExts('\\.sxi', 'application/vnd.sun.xml.impress');
-    $magic->addFileExts('\\.sxd', 'application/vnd.sun.xml.draw');
+
+    # FIXME: very ad hoc.
+    $magic->addFileExts('\\.sxw$', 'application/vnd.sun.xml.writer');
+    $magic->addFileExts('\\.sxc$', 'application/vnd.sun.xml.calc');
+    $magic->addFileExts('\\.sxi$', 'application/vnd.sun.xml.impress');
+    $magic->addFileExts('\\.sxd$', 'application/vnd.sun.xml.draw');
+
+    $magic->addFileExts('\\.odt$', 'application/vnd.oasis.opendocument.text');
+    $magic->addFileExts('\\.ods$', 'application/vnd.oasis.opendocument.spreadsheet');
+    $magic->addFileExts('\\.odp$', 'application/vnd.oasis.opendocument.presentation');
+    $magic->addFileExts('\\.odg$', 'application/vnd.oasis.opendocument.graphics');
     return;
 }
 
@@ -100,18 +111,23 @@ sub filter_metafile ($$$) {
     my ($contref, $weighted_str, $fields) = @_;
     my $metafile = 'meta.xml';
     my $xml = "";
-
-    my $tmpfile2  = util::tmpnam('NMZ.ooo2');
-    {
-        my $fh = util::efopen("> $tmpfile2");
+    my $tmpfile  = util::tmpnam('NMZ.zip');
+    { 
+        my $fh = util::efopen("> $tmpfile");
         print $fh $$contref;
+        util::fclose($fh);
     }
-
-    my $fh = util::efopen("$unzippath $unzipopts $tmpfile2 $metafile|");
-    while (defined(my $line = <$fh>)){
-        $xml .= $line;
-    }
-    unlink $tmpfile2;
+    my @cmd = ($unzippath, @unzipopts, $tmpfile, $metafile);
+    my $status = util::syscmd(
+        command => \@cmd,
+        option => {
+            "stdout" => \$xml,
+            "stderr" => "/dev/null",
+            "mode_stdout" => "wt",
+            "mode_stderr" => "wt",
+        },
+    );
+    unlink $tmpfile;
 
     my $authorname = ooo::get_author(\$xml);
     my $title = ooo::get_title(\$xml);
@@ -132,7 +148,6 @@ sub filter_metafile ($$$) {
     if (!($authorname eq "")){
         $fields->{'author'} = $authorname;
     }
-
     if (!($title eq "")){
         $fields->{'title'} = $title;
         my $weight = $conf::Weight{'html'}->{'title'};
@@ -149,17 +164,23 @@ sub filter_contentfile ($$$$$) {
     my ($contref, $weighted_str, $headings, $fields) = @_;
     my $contentfile = "content.xml";
     my $xml = "";
-    my $tmpfile2  = util::tmpnam('NMZ.ooo2');
-    {
-        my $fh = util::efopen("> $tmpfile2");
+    my $tmpfile  = util::tmpnam('NMZ.zip');
+    { 
+        my $fh = util::efopen("> $tmpfile");
         print $fh $$contref;
+        util::fclose($fh);
     }
-
-    my $fh = util::efopen("$unzippath $unzipopts $tmpfile2 $contentfile|");
-    while (defined(my $line = <$fh>)){
-        $xml .= $line;
-    }
-    unlink $tmpfile2;
+    my @cmd = ($unzippath, @unzipopts, $tmpfile, $contentfile);
+    my $status = util::syscmd(
+        command => \@cmd,
+        option => {
+            "stdout" => \$xml,
+            "stderr" => "/dev/null",
+            "mode_stdout" => "wt",
+            "mode_stderr" => "wt",
+        },
+    );
+    unlink $tmpfile;
 
     ooo::remove_all_tag(\$xml);
     ooo::decode_entity(\$xml);
@@ -215,18 +236,20 @@ sub utoe ($) {
     my ($tmp) = @_;
     if ($utfconvpath =~ /lv/){
         my $tmpfile  = util::tmpnam('NMZ.ooo');
-	{
-	    my $fh = util::efopen("> $tmpfile");
-	    print $fh $$tmp;
-	}
+        {
+            my $fh = util::efopen("> $tmpfile");
+            print $fh $$tmp;
+            util::fclose($fh);
+        }
         my $cmd = ($utfconvpath . " -Iu8 " . "-Oej " . $tmpfile . " |");
         $$tmp = "";
         my $fh = util::efopen($cmd);
         while (defined(my $line = <$fh>)){
             $$tmp .= $line;
         }
+        util::fclose($fh);
         unlink $tmpfile;
-    }elsif ($perlver >= 5.008){
+    }elsif ($English::PERL_VERSION >= 5.008){
         eval 'use Encode qw/from_to Unicode JP/;';
         Encode::from_to($$tmp, "utf-8" ,"euc-jp");
     }else{
