@@ -1,10 +1,11 @@
 # 
 # -*- Perl -*-
-# $Id: olertf.pl,v 1.4.4.3 2004/03/09 17:24:54 opengl2772 Exp $
+# $Id: olertf.pl,v 1.4.4.9 2007/01/14 04:12:04 opengl2772 Exp $
 # 
-# Copyright (C) 2000 Yoshinori.TAKESAKO ,
-#               2000 Jun Kurabe ,
-#               2000 Ken-ichi Hirose All rights reserved.
+# Copyright (C) 2000 Yoshinori.TAKESAKO,
+#               2000 Jun Kurabe,
+#               2000 Ken-ichi Hirose,
+#               2004-2007 Namazu Project All rights reserved.
 #     This is free software with ABSOLUTELY NO WARRANTY.
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -29,16 +30,23 @@ package olertf;
 use strict;
 require 'util.pl';
 require 'gfilter.pl';
+require 'olemsword.pl';
+
+use Win32::OLE::Const;
 
 sub mediatype() {
     return ('application/rtf');
 }
 
 sub status() {
+    # The check of a dependence filter.
+    return 'no' if (olemsword::status() ne 'yes');
+
     open (SAVEERR,">&STDERR");
     open (STDERR,">nul");
     my $const;
-    $const = Win32::OLE::Const->Load("Microsoft Word 11.0 Object Library");
+    $const = Win32::OLE::Const->Load("Microsoft Word 12.0 Object Library");
+    $const = Win32::OLE::Const->Load("Microsoft Word 11.0 Object Library") unless $const;
     $const = Win32::OLE::Const->Load("Microsoft Word 10.0 Object Library") unless $const;
     $const = Win32::OLE::Const->Load("Microsoft Word 9.0 Object Library") unless $const;
     $const = Win32::OLE::Const->Load("Microsoft Word 8.0 Object Library") unless $const;
@@ -56,13 +64,21 @@ sub pre_codeconv() {
 }
 
 sub post_codeconv () {
-    return 1;
+    return 0;
 }
 
 sub add_magic ($) {
     my ($magic) = @_;
 
+    $magic->addMagicEntry("0\tstring\t{\\rtf\t");
+    $magic->addMagicEntry(">6\tstring\t\\ansi\tapplication/rtf");
+    $magic->addMagicEntry(">6\tstring\t\\mac\tapplication/rtf");
+    $magic->addMagicEntry(">6\tstring\t\\pc\tapplication/rtf");
+    $magic->addMagicEntry(">6\tstring\t\\pca\tapplication/rtf");
+
+    # FIXME: very ad hoc.
     $magic->addFileExts('\\.rtf$', 'application/rtf');
+
     return;
 }
 
@@ -72,10 +88,8 @@ sub filter ($$$$$) {
 
     util::vprint("Processing rtf file ...\n");
 
-    $cfile =~ s/\//\\/g;
-    $$cont = "";
-    ReadMSWord::ReadMSWord($cfile, $cont, $fields, $weighted_str);
-    $cfile = defined $orig_cfile ? $$orig_cfile : '';
+    my $err = ReadMSWord::ReadDocument($cfile, $cont, $fields, $weighted_str);
+    return $err if (defined $err);
 
     gfilter::line_adjust_filter($cont);
     gfilter::line_adjust_filter($weighted_str);
