@@ -2,10 +2,10 @@
  * 
  * alias.c - 
  *
- * $Id: alias.c,v 1.15.8.3 2004/03/20 15:25:09 usu Exp $
+ * $Id: alias.c,v 1.15.8.5 2008/03/06 14:47:31 opengl2772 Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000,2001,2003 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2008 Namazu Project All rights reserved.
  * Copyright (C) 1999 NOKUBI Takatsugu All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
@@ -33,15 +33,22 @@
 #ifdef HAVE_SUPPORT_H
 #  include "support.h"
 #endif
+
 #include <stdio.h>
-#include <stdlib.h>
-#include <errno.h>
+#ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+#endif
+
+#ifdef HAVE_ERRNO_H
+#  include <errno.h>
+#endif
+
 #include <assert.h>
 
 #ifdef HAVE_STRING_H
-#include <string.h>
+#  include <string.h>
 #else
-#include <strings.h>
+#  include <strings.h>
 #endif
 
 #include "alias.h"
