@@ -1,6 +1,6 @@
 #
 # -*- Perl -*-
-# $Id: htmlsplit.pl,v 1.9 2001/01/10 08:42:52 knok Exp $
+# $Id: htmlsplit.pl,v 1.9.4.4 2004/05/23 10:25:17 opengl2772 Exp $
 #
 # Copyright (C) 2000 Namazu Project All rights reserved.
 #     This is free software with ABSOLUTELY NO WARRANTY.
@@ -24,7 +24,7 @@
 
 package htmlsplit;
 require "util.pl";
-require "html.pl";
+#require "html.pl"; # don't need it because it sould be already loaded by load_filtermodules()
 
 use strict;
 
@@ -72,6 +72,7 @@ sub split ($$) {
 
     my $id = 0;
 #    $cont =~ s/(<a\s[^>]*href=(["']))#(.+?)(\2[^>]*>)/$1$3.html$4/gi; #'
+    $cont =~ s#(<a[^>]*\s+)name=(["'])\2([^>]*>(.*?)</a>)#$1$4#sgi; #'
     $cont =~ s {
                 \G(.+?)                                      # 1
 	        (<h([1-6])>)?\s*                             # 2, 3
@@ -131,6 +132,9 @@ sub write_partial_file($$$$$$) {
     my $prev_name     = $info_ref->{'name'};
     my $prev_anchored = $info_ref->{'anchored'};
 
+    $prev_name        =~ s#\n\r##sg;
+    $prev_name        =~ s#\n##sg;
+
     html::remove_html_elements(\$prev_anchored);
     $prev_anchored =~ s/^\s+//;
     $prev_anchored =~ s/\s+$//;
@@ -162,7 +166,7 @@ sub write_partial_file($$$$$$) {
 
     # FIXME: Actually we don't need this. 
     #        But some perl versions need this.
-    close($fh);
+    util::fclose($fh);
     utime($mtime, $mtime, $fname);
 
     return "";
