@@ -1,6 +1,6 @@
 #
 # -*- Perl -*-
-# $Id: util.pl,v 1.22.4.13 2006/02/05 18:53:30 opengl2772 Exp $
+# $Id: util.pl,v 1.22.4.14 2007/08/09 18:39:50 opengl2772 Exp $
 # Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
 # Copyright (C) 2000-2006 Namazu Project All rights reserved.
 #     This is free software with ABSOLUTELY NO WARRANTY.
@@ -72,6 +72,8 @@ sub fclose ($) {
     if (ref $arg) {
         if ($arg =~ /^(IO::File|FileHandle)/) {
             my $fh = $arg;
+            $fh->flush;
+            cdie("write error: $!\n") if ($fh->error);
             $fh->close();
             return undef;
         }
