@@ -2,10 +2,10 @@
  * 
  * namazu.c - search client of Namazu
  *
- * $Id: namazu.c,v 1.102.8.6 2005/01/20 12:29:24 opengl2772 Exp $
+ * $Id: namazu.c,v 1.102.8.9 2007/12/05 16:59:36 opengl2772 Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000-2003 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2007 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -26,6 +26,10 @@
  * 
  */
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
@@ -34,10 +38,6 @@
 #include <sys/stat.h>
 #include <signal.h>
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
-
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #else
@@ -47,9 +47,9 @@
 #endif
 
 #ifdef HAVE_STRING_H
-#include <string.h>
+#  include <string.h>
 #else
-#include <strings.h>
+#  include <strings.h>
 #endif
 
 #include "namazu.h"
@@ -64,6 +64,9 @@
 #include "system.h"
 #include "namazu.h"
 #include "var.h"
+#include "charset.h"
+
+extern NMZ_HANDLE handle_charset;
 
 static char templatedir[BUFSIZE] = "";
 
@@ -108,6 +111,19 @@ make_fullpathname_msg(void)
  *
  */
 
+void
+exit_nmz(int status)
+{
+    /*
+     *
+     * free resource.
+     *
+     */
+    nmz_free_handle(handle_charset);
+
+    exit(status);
+}
+
 void 
 set_templatedir(char *dir)
 {
