@@ -1,9 +1,9 @@
 /*
- * $Id: result.c,v 1.59.8.21 2006/08/12 23:50:04 opengl2772 Exp $
+ * $Id: result.c,v 1.59.8.23 2007/12/05 16:59:36 opengl2772 Exp $
  * 
  * Copyright (C) 1989, 1990 Free Software Foundation, Inc.
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000-2006 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2007 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -23,8 +23,7 @@
  * 
  * 
  */
-#include <stdlib.h>
-#include <ctype.h>
+
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -32,10 +31,13 @@
 #  include "support.h"
 #endif
 
+#include <stdlib.h>
+#include <ctype.h>
+
 #ifdef HAVE_STRING_H
-#include <string.h>
+#  include <string.h>
 #else
-#include <strings.h>
+#  include <strings.h>
 #endif
 
 #include "namazu.h"
@@ -188,6 +190,12 @@ encode_entity(char *str)
 	    strncat(str, "&amp;", BUFSIZE - strlen(str) - 1);
 	} else if (tmp[i] == '"') {
 	    strncat(str, "&quot;", BUFSIZE - strlen(str) - 1);
+	} else if (tmp[i] == '\'') {
+	    strncat(str, "&#39;", BUFSIZE - strlen(str) - 1);
+	} else if (tmp[i] == '(') {
+	    strncat(str, "&#40;", BUFSIZE - strlen(str) - 1);
+	} else if (tmp[i] == ')') {
+	    strncat(str, "&#41;", BUFSIZE - strlen(str) - 1);
 	} else {
 	    if (strlen(str) < BUFSIZE - 1)
 		strncat(str, tmp + i, 1);
